#!/usr/bin/env python3

"""
Module entry point for ddn_metadata_bootstrap.
This allows the package to be executed with: python -m ddn_metadata_bootstrap
"""

from .main import main

if __name__ == "__main__":
    main()
